@@ -1,5 +1,5 @@
 /*
- * Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+ * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License").
  * You may not use this file except in compliance with the License.
@@ -17,7 +17,6 @@ plugins {
     `java-library`
     `maven-publish`
     signing
-    checkstyle
     jacoco
     id("com.github.spotbugs") version "4.7.4"
     id("io.codearte.nexus-staging") version "0.30.0"
@@ -197,14 +196,6 @@ subprojects {
             }
         }
 
-        /*
-         * CheckStyle
-         * ====================================================
-         */
-        apply(plugin = "checkstyle")
-
-        tasks["checkstyleTest"].enabled = false
-
         /*
          * Tests
          * ====================================================
