@@ -1,5 +1,5 @@
 /*
- * Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+ * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License").
  * You may not use this file except in compliance with the License.
@@ -589,6 +589,13 @@ public final class ShapeValueGenerator {
                 MemberShape member;
                 switch (currentShape.getType()) {
                     case STRUCTURE:
+                        // query-compatible error code override field isn't modeled, short-circuit here
+                        if (keyNode.getValue().equals("ErrorCodeOverride")) {
+                            var value = valueNode.expectStringNode().getValue();
+                            writer.write("ErrorCodeOverride: ptr.String($S),", value);
+                            break;
+                        }
+
                         if (currentShape.asStructureShape().get().getMember(keyNode.getValue()).isPresent()) {
                             member = currentShape.asStructureShape().get().getMember(keyNode.getValue()).get();
                         } else {
