@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+ * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License").
  * You may not use this file except in compliance with the License.
@@ -43,7 +43,10 @@ import software.amazon.smithy.model.SourceLocation;
 import software.amazon.smithy.rulesengine.language.Endpoint;
 import software.amazon.smithy.rulesengine.language.EndpointRuleSet;
 import software.amazon.smithy.rulesengine.language.error.RuleError;
+import software.amazon.smithy.rulesengine.language.evaluation.type.ArrayType;
 import software.amazon.smithy.rulesengine.language.evaluation.type.OptionalType;
+import software.amazon.smithy.rulesengine.language.evaluation.type.StringType;
+import software.amazon.smithy.rulesengine.language.evaluation.type.Type;
 import software.amazon.smithy.rulesengine.language.syntax.Identifier;
 import software.amazon.smithy.rulesengine.language.syntax.expressions.Expression;
 import software.amazon.smithy.rulesengine.language.syntax.expressions.ExpressionVisitor;
@@ -186,12 +189,18 @@ public final class EndpointResolverGenerator {
                                     continue;
                                 }
                                 switch (param.getType()) {
-                                    case STRING, BOOLEAN ->
+                                    case STRING, BOOLEAN -> {
                                         w.write("$L := *$L",
                                                 getLocalVarParameterName(param), getMemberParameterName(param));
+                                        // even if the parameter is required, it's not guaranteed that it will be used
+                                        // so we generate a blank identifier to prevent a compiler error if the
+                                        // variable is not used
+                                        w.write("_ = $L", getLocalVarParameterName(param));
+                                    }
                                     case STRING_ARRAY -> {
                                         w.write("$L := stringSlice($L)",
                                                 getLocalVarParameterName(param), getMemberParameterName(param));
+                                        w.write("_ = $L", getLocalVarParameterName(param));
                                     }
                                     default -> throw new CodegenException("unrecognized parameter type");
                                 }
@@ -304,14 +313,21 @@ public final class EndpointResolverGenerator {
         }
 
         if (fn.type() instanceof OptionalType || isConditionalFnResultOptional(condition, fn)) {
+            // []string (e.g. in endpoint params) needs to be casted as stringSlice instead of dereferenced for index
+            // operations
+            var isStringSlice = false;
+            if (fn.type() instanceof OptionalType opt) {
+                isStringSlice = isStringSlice(opt.inner());
+            }
             return goTemplate("""
                     if exprVal := $target:W; exprVal != nil {
-                        $conditionIdent:L := *exprVal
+                        $conditionIdent:L := $exprVal:L
                         _ = $conditionIdent:L
                         $next:W
                     }
                     """,
                     MapUtils.of(
+                            "exprVal", isStringSlice ? "stringSlice(exprVal)" : "*exprVal",
                             "conditionIdent", conditionIdentifier,
                             "target", generator.generate(fn),
                             "next", generateRule(
@@ -601,4 +617,11 @@ public final class EndpointResolverGenerator {
                     return &v
                 }""");
     }
+
+    private boolean isStringSlice(Type type) {
+        if (!(type instanceof ArrayType array)) {
+            return false;
+        }
+        return array.getMember() instanceof StringType;
+    }
 }
