@@ -1,5 +1,5 @@
 /*
- * Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+ * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License").
  * You may not use this file except in compliance with the License.
@@ -15,6 +15,8 @@
 
 package software.amazon.smithy.go.codegen.integration;
 
+import static software.amazon.smithy.go.codegen.GoWriter.goTemplate;
+import static software.amazon.smithy.go.codegen.SmithyGoDependency.SMITHY_TRACING;
 import static software.amazon.smithy.go.codegen.integration.ProtocolUtils.requiresDocumentSerdeFunction;
 
 import java.util.Collection;
@@ -250,6 +252,13 @@ public abstract class HttpBindingProtocolGenerator implements ProtocolGenerator
             writer.addUseImports(SmithyGoDependency.SMITHY);
             writer.addUseImports(SmithyGoDependency.SMITHY_HTTP_BINDING);
 
+            writer.write(goTemplate("""
+                    _, span := $T(ctx, "OperationSerializer")
+                    endTimer := startMetricTimer(ctx, "client.call.serialization_duration")
+                    defer endTimer()
+                    defer span.End()
+                    """, SMITHY_TRACING.func("StartSpan")));
+
             // cast input request to smithy transport type, check for failures
             writer.write("request, ok := in.Request.($P)", requestType);
             writer.openBlock("if !ok {", "}", () -> {
@@ -351,6 +360,8 @@ public abstract class HttpBindingProtocolGenerator implements ProtocolGenerator
             writer.write("in.Request = request");
             writer.write("");
 
+            writer.write("endTimer()");
+            writer.write("span.End()");
             writer.write("return next.$L(ctx, in)", generator.getHandleMethodName());
         });
     }
@@ -385,6 +396,13 @@ public abstract class HttpBindingProtocolGenerator implements ProtocolGenerator
             writer.write("if err != nil { return out, metadata, err }");
             writer.write("");
 
+            writer.write(goTemplate("""
+                    _, span := $T(ctx, "OperationDeserializer")
+                    endTimer := startMetricTimer(ctx, "client.call.deserialization_duration")
+                    defer endTimer()
+                    defer span.End()
+                    """, SMITHY_TRACING.func("StartSpan")));
+
             writer.write("response, ok := out.RawResponse.($P)", responseType);
             writer.openBlock("if !ok {", "}", () -> {
                 writer.addUseImports(SmithyGoDependency.SMITHY);
@@ -451,6 +469,7 @@ public abstract class HttpBindingProtocolGenerator implements ProtocolGenerator
             }
             writer.write("");
 
+            writer.write("span.End()");
             writer.write("return out, metadata, err");
         });
         goWriter.write("");
@@ -833,10 +852,8 @@ public abstract class HttpBindingProtocolGenerator implements ProtocolGenerator
                     });
         }
 
-        boolean allowZeroStrings = location != HttpBinding.Location.HEADER;
-
         GoValueAccessUtils.writeIfNonZeroValueMember(context.getModel(), context.getSymbolProvider(), writer,
-                memberShape, "v", allowZeroStrings, memberShape.isRequired(), (operand) -> {
+                memberShape, "v", true, memberShape.isRequired(), (operand) -> {
                     final String locationName = binding.getLocationName().isEmpty()
                             ? memberShape.getMemberName() : binding.getLocationName();
                     switch (location) {
@@ -845,6 +862,10 @@ public abstract class HttpBindingProtocolGenerator implements ProtocolGenerator
                             writeHeaderBinding(context, memberShape, operand, location, "locationName", "encoder");
                             break;
                         case PREFIX_HEADERS:
+                            // the prefix is allowed to be empty for headers, this simply means that there is none
+                            // and map keys are used directly as headers
+                            var prefix = binding.getLocationName();
+
                             MemberShape valueMemberShape = model.expectShape(targetShape.getId(),
                                     MapShape.class).getValue();
                             Shape valueMemberTarget = model.expectShape(valueMemberShape.getTarget());
@@ -855,14 +876,11 @@ public abstract class HttpBindingProtocolGenerator implements ProtocolGenerator
                                         + valueMemberShape.getId());
                             }
 
-                            writer.write("hv := encoder.Headers($S)", getCanonicalHeader(locationName));
+                            writer.write("hv := encoder.Headers($S)", getCanonicalHeader(prefix));
                             writer.addUseImports(SmithyGoDependency.NET_HTTP);
                             writer.openBlock("for mapKey, mapVal := range $L {", "}", operand, () -> {
-                                GoValueAccessUtils.writeIfNonZeroValue(context.getModel(), writer, valueMemberShape,
-                                        "mapVal", false, false, () -> {
-                                            writeHeaderBinding(context, valueMemberShape, "mapVal", location,
-                                                    "http.CanonicalHeaderKey(mapKey)", "hv");
-                                        });
+                                writeHeaderBinding(context, valueMemberShape, "mapVal", location,
+                                        "http.CanonicalHeaderKey(mapKey)", "hv");
                             });
                             break;
                         case LABEL:
@@ -964,6 +982,13 @@ public abstract class HttpBindingProtocolGenerator implements ProtocolGenerator
         }
 
         MemberShape collectionMemberShape = CodegenUtils.expectCollectionShape(targetShape).getMember();
+        // On empty collection header will be set to an empty value
+        writer.openBlock("if len($L) == 0 {", "}", operand, () -> {
+            writeHttpBindingSetter(context, writer, collectionMemberShape, location, operand,
+                    (w, s) -> {
+                        w.writeInline("$L.AddHeader($L).String(\"\")", dest, locationName);
+                    });
+        });
         writer.openBlock("for i := range $L {", "}", operand, () -> {
             // Only set non-empty non-nil header values
             String indexedOperand = operand + "[i]";
